@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -498,6 +498,38 @@ class Account(base.Base, authenticable.Authenticable):
         if send_email: account.email_new(password = password)
         return account
 
+    @classmethod
+    @appier.operation(
+        name = "Import CSV",
+        parameters = (
+            ("CSV File", "file", "file"),
+            ("Empty source", "empty", bool, False)
+        )
+    )
+    def import_csv_s(cls, file, empty):
+
+        def callback(line):
+            username,\
+            password,\
+            email,\
+            type = line
+
+            if type and type == "admin": type = cls.ADMIN_TYPE
+            else: type = cls.USER_TYPE
+
+            account = cls(
+                enabled = True,
+                username = username,
+                email = email,
+                password = password,
+                password_confirm = password,
+                type = type
+            )
+            account.save()
+
+        if empty: cls.delete_c()
+        cls._csv_import(file, callback)
+
     @classmethod
     def _build(cls, model, map):
         super(Account, cls)._build(model, map)
