@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -43,6 +43,7 @@ import time
 import random
 import string
 import hashlib
+import logging
 import datetime
 import collections
 
@@ -239,11 +240,12 @@ class Base(appier.Model):
         return False
 
     @classmethod
-    def secret_g(cls):
+    def secret_g(cls, hash = None):
+        hash = hash or hashlib.sha1
         token = "".join(random.choice(RANDOM_RANGE) for _index in range(32))
         token_bytes = appier.legacy.bytes(token)
-        url_sha1 = hashlib.sha1(token_bytes)
-        return url_sha1.hexdigest()
+        token_hash = hash(token_bytes)
+        return token_hash.hexdigest()
 
     @classmethod
     def build_index_g(cls, *args, **kwargs):
@@ -277,6 +279,7 @@ class Base(appier.Model):
         unsubscribe = kwargs.pop("unsubscribe", unsubscribe)
         logo = kwargs.pop("logo", logo)
         inline = kwargs.pop("inline", inline)
+        engine = kwargs.pop("engine", None)
         settings = dict(
             bulk = bulk,
             unsubscribe = unsubscribe,
@@ -286,7 +289,7 @@ class Base(appier.Model):
         if bulk: headers["Auto-Submitted"] = "auto-generated"
         if bulk: headers["Precedence"] = "bulk"
         if unsubscribe: headers["List-Unsubscribe"] = "<" + base_url + "/unsubscribe>"
-        html_handler = cls._inlinify
+        html_handler = (lambda data: cls._inlinify(data, engine = engine)) if engine else cls._inlinify
         html_handler = html_handler if inline else None
         owner.email(
             sender = sender,
@@ -446,13 +449,30 @@ class Base(appier.Model):
     def _inlinify_premailer(cls, data, *args, **kwargs):
         premailer = appier.import_pip("premailer")
         keep_style_tags = kwargs.get("keep_style_tags", True)
-        inliner = premailer.Premailer(data, keep_style_tags = keep_style_tags)
+        logging_level = kwargs.get(
+            "logging_level",
+            logging.INFO if appier.is_devel() else logging.ERROR
+        )
+        inliner = premailer.Premailer(
+            data,
+            keep_style_tags = keep_style_tags,
+            cssutils_logging_level = logging_level
+        )
         return inliner.transform(data)
 
     @classmethod
     def _inlinify_toronado(cls, data, *args, **kwargs):
         toronado = appier.import_pip("toronado")
-        return toronado.from_string(data)
+        encoding = kwargs.get("encoding", "utf-8")
+        logging_level = kwargs.get(
+            "logging_level",
+            logging.INFO if appier.is_devel() else logging.ERROR
+        )
+        toronado.logger.setLevel(logging_level)
+        result = toronado.from_string(data)
+        if appier.legacy.is_bytes(result):
+            result = result.decode(encoding)
+        return result
 
     def pre_create(self):
         appier.Model.pre_create(self)
@@ -617,9 +637,9 @@ class Base(appier.Model):
         kwargs["own"] = kwargs.pop("own", self)
         return cls.send_email_g(owner, *args, **kwargs)
 
-    def secret(self):
+    def secret(self, hash = None):
         cls = self.__class__
-        return cls.secret_g()
+        return cls.secret_g(hash = hash)
 
     @appier.operation(name = "Enable")
     def op_enable_s(self):
