@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -75,6 +75,23 @@ class Event(base.Base):
 
     @classmethod
     def transform(cls, arguments, prefix):
+        """
+        Runs the transformation process on the provided arguments
+        taking into account the filtering prefix, so that if the
+        argument name is prefixed by the provided prefix such prefix
+        is removed on a new re-set arguments.
+
+        :type arguments: Dictionary
+        :param arguments: The dictionary of arguments for which the
+        prefix transformation is going to take place.
+        :type prefix: String
+        :param prefix: The prefix that is going to be used in the
+        prefix removal transformation process.
+        :rtype: Dictionary
+        :return: The new dictionary of arguments with the processed
+        values without the associated prefix set.
+        """
+
         arguments = dict(arguments)
         prefix_l = len(prefix)
         for key, value in appier.legacy.items(arguments):
@@ -83,6 +100,16 @@ class Event(base.Base):
             arguments[key_prefix] = value
         return arguments
 
+    @classmethod
+    def format(cls, arguments, all = False):
+        arguments = dict(arguments)
+        for key, value in appier.legacy.items(arguments):
+            if not appier.legacy.is_string(value, all = all): continue
+            try: value = value.format(**arguments)
+            except: value = value
+            arguments[key] = value
+        return arguments
+
     @classmethod
     def notify_g(cls, name, handlers = None, arguments = {}):
         logger = appier.get_logger()
@@ -106,6 +133,7 @@ class Event(base.Base):
         arguments_m.update(arguments)
         arguments_m.update(event = self.name, handler = self.handler)
         arguments_m = cls.transform(arguments_m, self.handler + "_")
+        arguments_m = cls.format(arguments_m)
         kwargs = dict(arguments = arguments_m)
         if delay: self.owner.delay(method, kwargs = kwargs)
         else: method(arguments, **kwargs)
@@ -143,30 +171,49 @@ class Event(base.Base):
             absolute = absolute
         )
 
+    @classmethod
+    def _retry(cls, callable, count = 3):
+        result = None
+        while True:
+            try:
+                result = callable()
+            except:
+                count -= 1
+                if count == 0: raise
+                else: continue
+            break
+        return result
+
     def notify_http(self, arguments = {}):
+        cls = self.__class__
         url = arguments.get("url", None)
+        retries = arguments.get("retries", 3)
         logger = appier.get_logger()
         logger.debug("Running HTTP notification for '%s' ..." % url)
-        return appier.post(url, data_j = arguments)
+        return cls._retry(lambda: appier.post(url, data_j = arguments), count = retries)
 
     def notify_mailme(self, arguments = {}):
+        cls = self.__class__
         appier.ensure_pip("mailme", package = "mailme_api")
         import mailme
+        retries = arguments.get("retries", 3)
         logger = appier.get_logger()
         logger.debug("Running Mailme notification ...")
         api = mailme.API()
-        return api.send(arguments)
+        return cls._retry(lambda: api.send(arguments), count = retries)
 
     def notify_nexmo(self, arguments = {}):
+        cls = self.__class__
         appier.ensure_pip("nexmo", package = "nexmo_api")
         import nexmo
         sender = arguments["sender"]
         receiver = arguments["receiver"]
         text = arguments["text"]
+        retries = arguments.get("retries", 3)
         logger = appier.get_logger()
         logger.debug("Running Nexmo notification for '%s' ..." % receiver)
         api = nexmo.API()
-        return api.send_sms(sender, receiver, text)
+        return cls._retry(lambda: api.send_sms(sender, receiver, text), count = retries)
 
     @appier.operation(name = "Duplicate", factory = True)
     def duplicate_s(self):
