@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -91,8 +91,18 @@ class AdminPart(
         self.style = appier.conf("ADMIN_STYLE", self.style)
         self.libs = appier.conf("ADMIN_LIBS", self.libs)
         self.social_libs = appier.conf("ADMIN_SOCIAL_LIBS", self.social_libs, cast = list)
+        self.available = appier.conf("ADMIN_AVAILABLE", self.available, cast = bool)
+        self.open = appier.conf("ADMIN_OPEN", self.open, cast = bool)
+        self.oauth = appier.conf("ADMIN_OAUTH", self.oauth, cast = bool)
+        self.avatar_default = appier.conf(
+            "ADMIN_AVATAR_DEFAULT",
+            self.avatar_default,
+            cast = bool
+        )
+
         self._last_login = None
         self._login_count = 0
+        self._sections = appier.OrderedDict()
 
     def version(self):
         return base.VERSION
@@ -130,21 +140,14 @@ class AdminPart(
         self.owner.admin_google_scope = ("email",)
         self.owner.admin_live_scope = ("wl.basic", "wl.emails")
 
-        self.owner.admin_available = appier.conf(
-            "ADMIN_AVAILABLE", self.available, cast = bool
-        )
-        self.owner.admin_open = appier.conf(
-            "ADMIN_OPEN", self.open, cast = bool
-        )
-        self.owner.admin_oauth = appier.conf(
-            "ADMIN_OAUTH", self.oauth, cast = bool
-        )
-        self.owner.admin_avatar_default = appier.conf(
-            "ADMIN_AVATAR_DEFAULT", self.avatar_default, cast = bool
-        )
+        self.owner.admin_available = self.available
+        self.owner.admin_open = self.open
+        self.owner.admin_oauth = self.oauth
+        self.owner.admin_avatar_default = self.avatar_default
 
         self.owner.lib_loaders["appier_extras"] = self._appier_extras_loader
         self.owner.lib_loaders["netius"] = self._netius_loader
+        self.owner.lib_loaders["pconvert"] = self._pconvert_loader
         self.owner.lib_loaders["PIL"] = self._pil_loader
         self.owner.lib_loaders["pymongo"] = self._pymongo_loader
         self.owner.lib_loaders["redis"] = self._redis_loader
@@ -196,6 +199,8 @@ class AdminPart(
             (("GET",), "/admin/configs", self.list_configs),
             (("GET",), "/admin/parts", self.list_parts),
             (("GET",), "/admin/parts/<str:name>", self.show_part),
+            (("GET",), "/admin/parts/<str:name>/load", self.load_part),
+            (("GET",), "/admin/parts/<str:name>/unload", self.unload_part),
             (("GET",), "/admin/libraries", self.list_libraries),
             (("GET",), "/admin/oauth/authorize", self.oauth_authorize),
             (("POST",), "/admin/oauth/authorize", self.do_oauth_authorize),
@@ -264,12 +269,15 @@ class AdminPart(
             (("GET",), "/admin/configs.json", self.show_configs, None, True),
             (("GET",), "/api/admin/ping", self.ping_api, None, True),
             (("GET", "POST"), "/api/admin/login", self.login_api, None, True),
+            (("GET", "POST"), "/api/admin/oauth/access_token", self.oauth_access_token_api, None, True),
+            (("GET", "POST"), "/api/admin/oauth/login", self.oauth_login_api, None, True),
             (("GET",), "/api/admin/accounts/me", self.me_account_api, None, True),
             (("GET",), "/api/admin/models/<str:model>", self.show_model_json, None, True),
             (("GET",), "/api/admin/models/<str:model>/<str:_id>", self.show_entity_json, None, True)
         ]
 
     def models(self):
+        if not self.available: return None
         return models
 
     def template(self, template, layout = None, *args, **kwargs):
@@ -278,7 +286,6 @@ class AdminPart(
         return appier.Part.template(
             self,
             template,
-            models_d = self.models_d,
             *args,
             **kwargs
         )
@@ -321,6 +328,22 @@ class AdminPart(
     def flush_settings(self):
         self.dump_settings()
 
+    def add_section(self, name):
+        self._sections[name] = appier.OrderedDict()
+
+    def remove_section(self, name):
+        del self._sections[name]
+
+    def add_section_item(self, name, route, section = None):
+        if not section in self._sections:
+            self.add_section(section)
+        self._sections[section][name] = route
+
+    def remove_section_item(self, name, section = None):
+        del self._sections[section][name]
+        if self._sections[section]: return
+        del self._sections[section]
+
     def index(self):
         return self.list_models()
 
@@ -497,7 +520,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure()
+    @appier.ensure(context = "admin")
     def me_account(self):
         account_c = self._get_cls(self.account_c)
         account = account_c.from_session(meta = True)
@@ -506,7 +529,7 @@ class AdminPart(
             account = account
         )
 
-    @appier.ensure("admin.accounts")
+    @appier.ensure(token = "admin.accounts", context = "admin")
     def show_account(self, username):
         account_c = self._get_cls(self.account_c)
         account = account_c.get(
@@ -518,7 +541,7 @@ class AdminPart(
             account = account
         )
 
-    @appier.ensure("admin.accounts")
+    @appier.ensure(token = "admin.accounts", context = "admin")
     def mail_account(self, username):
         raise appier.NotImplementedError()
 
@@ -535,7 +558,7 @@ class AdminPart(
             cache = cache
         )
 
-    @appier.ensure(token = "admin.options")
+    @appier.ensure(token = "admin.options", context = "admin")
     def options(self):
         return self.template(
             "options.html.tpl",
@@ -544,7 +567,7 @@ class AdminPart(
             errors = dict()
         )
 
-    @appier.ensure(token = "admin.options")
+    @appier.ensure(token = "admin.options", context = "admin")
     def options_action(self):
         layout = self.field("layout")
         theme = self.field("theme")
@@ -580,7 +603,7 @@ class AdminPart(
             errors = dict()
         )
 
-    @appier.ensure(token = "admin.social")
+    @appier.ensure(token = "admin.social", context = "admin")
     def social(self):
         socials = self.socials()
         linked = self.linked()
@@ -591,21 +614,21 @@ class AdminPart(
             linked = linked
         )
 
-    @appier.ensure(token = "admin.operations")
+    @appier.ensure(token = "admin.operations", context = "admin")
     def operations(self):
         return self.template(
             "operations.html.tpl",
             section = "operations"
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def status(self):
         return self.template(
             "status.html.tpl",
             section = "status"
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_routes(self):
         return self.template(
             "routes.html.tpl",
@@ -613,7 +636,7 @@ class AdminPart(
             routes = self._routes()
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_configs(self):
         configs = appier.config.CONFIGS
         configs = appier.legacy.items(configs)
@@ -624,7 +647,7 @@ class AdminPart(
             configs = configs
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_parts(self):
         parts = self.owner.get_parts()
         return self.template(
@@ -633,7 +656,7 @@ class AdminPart(
             parts = parts
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def show_part(self, name):
         part = self.owner.get_part(name)
         info = part.info()
@@ -647,7 +670,31 @@ class AdminPart(
             info_l = info_l
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
+    def load_part(self, name):
+        part = self.owner.get_part(name)
+        self.owner._load_part(part)
+        return self.redirect(
+            self.url_for(
+                "admin.show_part",
+                name = name,
+                message = "Part %s has been loaded" % name
+            )
+        )
+
+    @appier.ensure(token = "admin.status", context = "admin")
+    def unload_part(self, name):
+        part = self.owner.get_part(name)
+        self.owner._unload_part(part)
+        return self.redirect(
+            self.url_for(
+                "admin.show_part",
+                name = name,
+                message = "Part %s has been unloaded" % name
+            )
+        )
+
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_libraries(self):
         libraries = self.owner.get_libraries()
         return self.template(
@@ -656,7 +703,7 @@ class AdminPart(
             libraries = libraries
         )
 
-    @appier.private
+    @appier.ensure(context = "admin")
     def oauth_authorize(self):
         # verifies if the oauth system is allowed if that's not
         # the case raises an exception indicating so
@@ -716,7 +763,7 @@ class AdminPart(
             tokens = tokens
         )
 
-    @appier.ensure()
+    @appier.ensure(context = "admin")
     def do_oauth_authorize(self):
         if not self.owner.admin_oauth: raise appier.SecurityError(
             message = "OAuth not allowed"
@@ -749,7 +796,7 @@ class AdminPart(
             )
         )
 
-    @appier.private
+    @appier.ensure(context = "admin")
     def oauth_deny(self):
         if not self.owner.admin_oauth: raise appier.SecurityError(
             message = "OAuth not allowed"
@@ -800,12 +847,16 @@ class AdminPart(
         oauth_token.unset_code_s()
 
         # returns the final map based response containing the complete
-        # set of elements to the client to be used
+        # set of elements to the client to be used, notice that both
+        # the scope and the tokens are also returned so that the OAuth
+        # client is able to modify experience taking that into account
         return dict(
             access_token = oauth_token.access_token,
             token_type = "bearer",
             expires_in = oauth_token.expires_in,
-            refresh_token = oauth_token.refresh_token
+            refresh_token = oauth_token.refresh_token,
+            scope = oauth_token.scope,
+            tokens = oauth_token.tokens
         )
 
     def oauth_login(self):
@@ -832,7 +883,7 @@ class AdminPart(
         # values to correctly authenticate the user
         oauth_token._set_session()
 
-        # retrieves the session identifier (sid) for the currently
+        # retrieves the session identifier (SID) for the currently
         # assigned session, this is going to be used in the next
         # requests to refer to the proper session
         sid = self.session.sid
@@ -846,7 +897,7 @@ class AdminPart(
             tokens = oauth_token.tokens
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def build_index(self):
         empty = self.field("empty", True, cast = bool)
         if empty: models.Search.delete_c()
@@ -859,7 +910,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def build_index_db(self):
         for model in self._administrable(self.models_r):
             model._destroy_indexes()
@@ -871,7 +922,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def test_email(self):
         receiver = appier.conf("TEST_EMAIL", None)
         receiver = self.field("email", receiver)
@@ -893,7 +944,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def test_event(self):
         name = appier.conf("TEST_EVENT", "test")
         name = self.field("event", name)
@@ -906,7 +957,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_sessions(self):
         return self.template(
             "sessions.html.tpl",
@@ -914,7 +965,7 @@ class AdminPart(
             sessions = self.request.session_c.all()
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def empty_sessions(self):
         self.request.session_c.empty()
         return self.redirect(
@@ -924,7 +975,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def show_session_me(self):
         sid = self.session.sid
         return self.template(
@@ -943,7 +994,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def show_session(self, sid):
         sid = str(sid)
         return self.template(
@@ -952,7 +1003,7 @@ class AdminPart(
             session_s = self.request.session_c.get_s(sid)
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def delete_session(self, sid):
         sid = str(sid)
         self.request.session_c.expire(sid)
@@ -963,7 +1014,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin.status")
+    @appier.ensure(token = "admin.status", context = "admin")
     def list_counters(self):
         collection = self._counters()
         counters = collection.find()
@@ -973,7 +1024,7 @@ class AdminPart(
             counters = counters
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def list_events_csv(self):
         object = appier.get_object(
             alias = True,
@@ -999,14 +1050,14 @@ class AdminPart(
         self.content_type("text/csv")
         return result
 
-    @appier.ensure(token = "admin.database")
+    @appier.ensure(token = "admin.database", context = "admin")
     def database(self):
         return self.template(
             "database.html.tpl",
             section = "database"
         )
 
-    @appier.ensure(token = "admin.database")
+    @appier.ensure(token = "admin.database", context = "admin")
     def database_export(self):
         adapter = self.get_adapter()
         file = appier.legacy.BytesIO()
@@ -1028,14 +1079,14 @@ class AdminPart(
 
         return file.getvalue()
 
-    @appier.ensure(token = "admin.database")
+    @appier.ensure(token = "admin.database", context = "admin")
     def database_import(self):
         return self.template(
             "database/import.html.tpl",
             section = "database"
         )
 
-    @appier.ensure(token = "admin.database")
+    @appier.ensure(token = "admin.database", context = "admin")
     def database_import_do(self):
         # tries to retrieve the reference to the import file tuple
         # and in case it's not found raises an error to the template
@@ -1069,7 +1120,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin.database")
+    @appier.ensure(token = "admin.database", context = "admin")
     def database_reset(self):
         adapter = self.get_adapter()
         adapter.drop_db()
@@ -1080,7 +1131,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def search(self):
         object = appier.get_object(
             alias = True,
@@ -1091,14 +1142,14 @@ class AdminPart(
         indexes = models.Search.find(map = True, **object)
         return indexes
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def list_models(self):
         return self.template(
             "models/list.html.tpl",
             section = "admin"
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_model(self, model):
         appier.ensure_login(self, token = "admin.models." + model)
         model = self.get_model(model, raise_e = True)
@@ -1119,7 +1170,7 @@ class AdminPart(
             entities = entities
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_model_json(self, model):
         appier.ensure_login(self, token = "admin.models." + model)
         eager_l = self.field("eager_l", False, cast = bool)
@@ -1134,7 +1185,7 @@ class AdminPart(
         )
         return entities
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_model_csv(self, model):
         appier.ensure_login(self, token = "admin.models." + model)
         eager_l = self.field("eager_l", False, cast = bool)
@@ -1151,7 +1202,7 @@ class AdminPart(
         self.content_type("text/csv")
         return result
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def link_model(self, model, link):
         # ensures that the proper token is available so that the
         # operation may be executed under the current context
@@ -1215,7 +1266,7 @@ class AdminPart(
         # result string (and URL) value
         return self.redirect(result)
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def operation_model(self, model, operation):
         # ensures that the proper token is available so that the
         # operation may be executed under the current context
@@ -1290,7 +1341,7 @@ class AdminPart(
             return self.redirect(
                 self.url_for(
                     "admin.show_entity",
-                    message = "Operation '%s' completed with success" % operation_s,
+                    message = "Operation %s completed with success" % operation_s,
                     model = model_name,
                     _id = model_id
                 )
@@ -1301,13 +1352,13 @@ class AdminPart(
         return self.redirect(
             next or self.url_for(
                 "admin.show_model",
-                message = "Operation '%s' completed with success" % operation_s,
+                message = "Operation %s completed with success" % operation_s,
                 model = model._under()
             ),
-            message = "Operation '%s' completed with success" % operation_s
+            message = "Operation %s completed with success" % operation_s
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def view_model(self, model, view):
         appier.ensure_login(self, token = "admin.models." + model)
 
@@ -1362,7 +1413,7 @@ class AdminPart(
             is_global = is_global
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def new_entity(self, model):
         model = self.get_model(model)
         return self.template(
@@ -1373,7 +1424,7 @@ class AdminPart(
             model = model
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def create_entity(self, model):
         model = self.get_model(model)
         entity = model.new(safe = False)
@@ -1391,7 +1442,7 @@ class AdminPart(
             self.url_for("admin.show_model", model = model._under())
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
         model = self.get_model(model, raise_e = True)
@@ -1410,7 +1461,7 @@ class AdminPart(
             next_url = next_url
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_entity_json(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
         eager_l = self.field("eager_l", True, cast = bool)
@@ -1426,7 +1477,7 @@ class AdminPart(
         )
         return entity
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def edit_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
         model = self.get_model(model, raise_e = True)
@@ -1443,7 +1494,7 @@ class AdminPart(
             model = model
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def update_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
         model = self.get_model(model, raise_e = True)
@@ -1471,7 +1522,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def delete_entity(self, model, _id):
         appier.ensure_login(self, token = "admin.models." + model)
         model = self.get_model(model, raise_e = True)
@@ -1785,7 +1836,7 @@ class AdminPart(
            next or self.url_for(self.owner.admin_login_redirect)
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_log(self):
         memory_handler = self.owner.handler_memory
         count = self.field("count", 100, cast = int)
@@ -1794,13 +1845,19 @@ class AdminPart(
             messages = memory_handler.get_latest(count = count, level = level)
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin", context = "admin")
     def show_configs(self):
         return appier.config.CONFIGS
 
     def ping_api(self):
         return dict(time = time.time())
 
+    def oauth_access_token_api(self):
+        return self.oauth_access_token()
+
+    def oauth_login_api(self):
+        return self.oauth_login()
+
     def login_api(self):
         # verifies if the current administration interface is
         # available and if that's not the cases raises an error
@@ -1819,7 +1876,7 @@ class AdminPart(
         # values to correctly authenticate the user
         account._set_session()
 
-        # retrieves the session identifier (sid) for the currently
+        # retrieves the session identifier (SID) for the currently
         # assigned session, this is going to be used in the next
         # requests to refer to the proper session
         sid = self.session.sid
@@ -1833,7 +1890,7 @@ class AdminPart(
             tokens = account.tokens()
         )
 
-    @appier.ensure()
+    @appier.ensure(context = "admin")
     def me_account_api(self):
         account_c = self._get_cls(self.account_c)
         account = account_c.from_session(map = True)
@@ -1870,7 +1927,7 @@ class AdminPart(
             ),
             encoding = encoding
         )
-        return appier.legacy.str(
+        return appier.legacy.u(
             value,
             encoding = encoding,
             force = True
@@ -1900,6 +1957,12 @@ class AdminPart(
             versions.append(("Netius", module.VERSION))
         return versions
 
+    def _pconvert_loader(self, module):
+        versions = []
+        if hasattr(module, "VERSION"):
+            versions.append(("P(NG)Convert", module.VERSION))
+        return versions
+
     def _pil_loader(self, module):
         versions = []
         if hasattr(module, "VERSION"):
@@ -2109,7 +2172,10 @@ class AdminPart(
         method_kw = appier.legacy.getargspec(method)[2]
         result = method(**parameters_kw) if method_kw else method(*parameters, **kwargs)
 
-        appier.verify(not cls or result["model"] == cls)
+        # ensures that the model type in the result from the view apply
+        # is the expected one (if entity class is provided) this is relevant
+        # to avoid issues with bad filtering methods
+        appier.verify(not cls or result["model"].is_equal(cls))
 
         kwargs = result["kwargs"]
         return kwargs
